---------------------------------------------------------------------------------------------------
--[[
		Add-on by Durveg. Source code taken from Carbine.
]]
---------------------------------------------------------------------------------------------------

require "Window"
require "DialogSys"
require "Quest"
require "DialogResponse"
require "QuestLib"
require "GameLib"
require "ChatSystemLib"

local InstantQuestAccept = {}

-- TODO Hardcoded Colors for Items
local karEvalColors = 
{
	[Item.CodeEnumItemQuality.Inferior] 		= ApolloColor.new("ItemQuality_Inferior"),
	[Item.CodeEnumItemQuality.Average] 			= ApolloColor.new("ItemQuality_Average"),
	[Item.CodeEnumItemQuality.Good] 			= ApolloColor.new("ItemQuality_Good"),
	[Item.CodeEnumItemQuality.Excellent] 		= ApolloColor.new("ItemQuality_Excellent"),
	[Item.CodeEnumItemQuality.Superb] 			= ApolloColor.new("ItemQuality_Superb"),
	[Item.CodeEnumItemQuality.Legendary] 		= ApolloColor.new("ItemQuality_Legendary"),
	[Item.CodeEnumItemQuality.Artifact]		 	= ApolloColor.new("ItemQuality_Artifact"),
}

local kcrDefaultOptionColor = ApolloColor.new("white")
local kcrHighlightOptionColor = ApolloColor.new(110/255, 255/255, 72/255, 1.0)

function InstantQuestAccept:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	return o
end

function InstantQuestAccept:Init()
	Apollo.RegisterAddon(self, true)
end

---------------------------------------------------------------------------------------------------
-- Dialog EventHandlers
---------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnLoad()
	Apollo.LoadSprites("UI\\Dialog\\DialogSprites.xml") -- Old
	Apollo.RegisterEventHandler("Dialog_ShowState", "OnDialog_ShowState", self)
	Apollo.RegisterEventHandler("Dialog_Close", "OnDialog_Close", self)
	
	Apollo.RegisterTimerHandler("StartError", "Error", self)
	
	self.DefaultQuest = Apollo.GetAddon("Dialog")
	self.DefaultQuestNum = 1
	if self.DefaultQuest == nil then
		self.DefaultQuest = Apollo.GetAddon("UnitedDialogs")
		self.DefaultQuestNum = 3
	end
	
	if self.DefaultQuest == nil then
		self.DefaultQuest = Apollo.GetAddon("ClassicQuestDialog")
		self.DefaultQuestNum = 2
		if self.DefaultQuest.AddonVersion == nil then
			Apollo.AddAddonErrorText(self, "If you wish to use ClassicQuestDialog for the quest turn in work around, please update it to the latest version.")
			Apollo.CreateTimer("StartError", 1, false)
			return
		end
	end
	
	self.Conf = Apollo.LoadForm("InstantQuestAccept.xml", "Configure", nil, self) 
	
	self.wndMain = Apollo.LoadForm("InstantQuestAccept.xml", "MainWindow", nil, self)
	self.wndMain:ToFront()
	
	local tMax = Apollo.GetDisplaySize()
	self.wndMain:SetAnchorOffsets(tMax.nWidth - 450, 50, tMax.nWidth - 50, 650)
		
	Apollo.RegisterTimerHandler("HideWindow", "HideWindow", self)

	
	self.bRewardPicked = false
	
	self.PostLog = true
	
	-- To Change default setting, both of these must be false, else will only change after opening the config window
	self.AcceptNow = true -- If true keep "Pressing" Accept
	self.AcceptAllSetting = true -- Setting Modifier, if true will accept all quests from an npc on interaction
end

function InstantQuestAccept:Error()
	--Simple way to create an error because I am too lazy to figure out the right way.
	self.wndMain:Show(true)
end

function InstantQuestAccept:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
        return nil
    end

	local locWindowLocation = self.wndMain and self.wndMain:GetLocation() or self.locSavedWindowLoc
	
    local tSave =
	{
		tWindowLocation = locWindowLocation and locWindowLocation:ToTable() or nil,
		accAll = self.AcceptAllSetting,
		Post = self.PostLog,
	}

    return tSave
end

function InstantQuestAccept:OnRestore(eType, tSave)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	end

	if tSave.accAll ~= nil then
		self.AcceptAllSetting = tSave.accAll
	end

	if self.AcceptAllSetting == true then
		self.AcceptNow = true
	elseif self.AcceptAllSetting == false then
		self.AcceptNow = false
	end
	
	if tSave.Post == false then
		self.PostLog = false
	else
		self.PostLog = true --tSave.Post
	end
	
	if tSave.tWindowLocation then
		self.locSavedWindowLoc = WindowLocation.new(tSave.tWindowLocation)
		
		if self.wndMain then
			self.wndMain:MoveToLocation(self.locSavedWindowLoc)
		end
	end
end

---------------------------------------------------------------------------------------------------
-- Settings window 
---------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnConfigure()
	self.Conf:Show(true)
	self.Conf:FindChild("AcceptAllSetting"):SetCheck(self.AcceptAllSetting)
	self.Conf:FindChild("SendToChat"):SetCheck(self.PostLog)
end

---------------------------------------------------------------------------------------------------
-- On any close of the window
---------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnDialog_Close() --Closed on CloseButton Press or ESC
	
	if self.AcceptAllSetting == false then
		self.AcceptNow = false
	else
		self.AcceptNow = true
	end
	
	self.wndMain:Close()
	Apollo.StopTimer("HideWindow")
end

---------------------------------------------------------------------------------------------------
-- On any click of a NPC with a Quest 
---------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnDialog_ShowState(eState, queCurrent)

	local DontAuto = false
	if self.DefaultQuestNum == 1 and (self.DefaultQuest.wndPlayer:IsShown() or self.DefaultQuest.wndNpc:IsShown() or
  	 self.DefaultQuest.wndItem:IsShown()) then
		DontAuto = true
	elseif self.DefaultQuestNum == 2 and self.DefaultQuest.wndMain:IsShown() then
		DontAuto = true
	elseif self.DefaultQuestNum == 3 and (self.DefaultQuest.wndPlayer:IsShown() or self.DefaultQuest.wndNpc:IsShown() or
  	 self.DefaultQuest.wndItem:IsShown()) then
		DontAuto = true
	end
	
	if not (Apollo.IsShiftKeyDown() and Apollo.IsAltKeyDown() or DontAuto) then
		if self.DefaultQuestNum == true then
			self.DefaultQuest.InstantCalled = true
		end
		Apollo.CreateTimer("HideWindow", .1, true)
		self.LastID = nil
		
		local idQuest = 0
		local strQuestTitle = ""
		local strQuestLevel = ""
		
		if queCurrent and queCurrent:GetId() then
			idQuest = queCurrent:GetId()

			if string.len(queCurrent:GetTitle()) > 0 then
				strQuestTitle = queCurrent:GetTitle()
				strQuestLevel = tostring(queCurrent:GetConLevel())
			end	
		end
		
		self.bRewardPicked = false

		if eState == DialogSys.DialogState_Inactive or
			eState == DialogSys.DialogState_Vending or
			eState == DialogSys.DialogState_Training or
			eState == DialogSys.DialogState_TradeskillTraining or
			eState == DialogSys.DialogState_CraftingStation then

			self:OnDialog_Close() -- Close if they click vending/training, as we open another window
			return
		end

		local tResponseList = DialogSys.GetResponses(idQuest)
		if not tResponseList or #tResponseList == 0 then
			self:OnDialog_Close()
			return
		end
		
		-- Player Window
		self:DrawResponses(eState, idQuest, tResponseList)

		-- DialogSys.GetNPC() is nill or :GetName() will throw an error
		if DialogSys.GetNPC() ~= nil then 
			-- if the player is looking at the EXP buffs on the housing board, don't auto choose a buff 
			if 	DialogSys.GetNPC():GetName() == "Housing Bonus Board" and self.wndMain:FindChild("Accept"):GetData():GetText() ~= "I want this bonus today!" or 
			    DialogSys.GetNPC():GetId() == 355514 and self.wndMain:FindChild("Accept"):GetData():GetText() ~= "I want this bonus today!" then
				self.AcceptNow = false
			end
		end
	
		if self.AcceptNow == true and self.wndMain:FindChild("Accept"):GetText() ~= "Not used" then
			if self.wndMain:FindChild("Accept"):GetText() == "Complete" and strQuestTitle ~= "" and self.PostLog == true then
				if self.LastPrinted ~= "Quest Completed: " .. strQuestTitle then
					self.LastPrinted = "Quest Completed: " .. strQuestTitle
					ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, self.LastPrinted) 
				end
			elseif self.wndMain:FindChild("Accept"):GetText() == "Accept" and strQuestTitle ~= "" and self.PostLog == true then
				if self.LastPrinted ~= "Quest Accepted: " .. strQuestTitle .. ", Quest Level: " .. strQuestLevel then
					self.LastPrinted = "Quest Accepted: " .. strQuestTitle .. ", Quest Level: " .. strQuestLevel
					ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, self.LastPrinted)
				end
			end
		
			self.wndMain:FindChild("Accept"):GetData():Select()-- "Press" accept
		
			if self.wndMain:FindChild("Accept"):GetText() == "Accept" then
				self.wndMain:Show(false)
			end
		
			-- Reset AcceptNow after accepting a quest if AcceptAllSetting option is off 
			if DialogSys.GetCommCreatureId() == nil then 
				if DialogSys.GetNPC() ~= nil then 
					-- if the player is looking at the EXP buffs on the housing board, don't auto choose a buff 
					if  self.wndMain:FindChild("Accept"):GetText() == "Complete" and DialogSys.GetNPC():GetName() == "Housing Bonus Board" and self.AcceptAllSetting == false then 
						self.AcceptNow = false
					end
				end
			end
		
			-- Reset AcceptNow after accepting a quest if AcceptAllSetting option is off 
			if self.wndMain:FindChild("Accept"):GetText() == "Accept" and self.AcceptAllSetting == false then 
				self.AcceptNow = false
			end	
		end
	end
end

function InstantQuestAccept:HideWindow()
	if self.DefaultQuestNum == 1 or self.DefaultQuestNum == 3 then
		self.DefaultQuest.wndPlayer:Show(false)
		self.DefaultQuest.wndNpc:Show(false)
		self.DefaultQuest.wndItem:Show(false)
	elseif self.DefaultQuestNum == 2 then
		self.DefaultQuest.wndMain:Show(false)
	end
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:DrawResponses(eState, idQuest, tResponseList)-- Draws Player Dialog Window	Print("InDraw")
	-- Quest Selection Window	
	local SelectWnd = self.wndMain:FindChild("Scroll")
	
	self.wndMain:FindChild("Accept"):SetTooltip("")
	self.wndMain:FindChild("Accept"):SetData(nil)

	self.wndMain:FindChild("Decline"):SetTooltip("")
	self.wndMain:FindChild("Decline"):SetData(nil)
	
	SelectWnd:FindChild("ResponseItemContainer"):DestroyChildren()
	
	self.wndMain:FindChild("Accept_Decline"):FindChild("Accept"):Show(false)
	self.wndMain:FindChild("Accept_Decline"):FindChild("Decline"):Show(false)
	self.wndMain:FindChild("Accept_Decline"):FindChild("Conditional"):Show(false)
	
	local queCurr = DialogSys.GetViewableQuest(idQuest)
	local NumChoice = 0
	local First_drResponse = nil
	local IncompleteOnly = true
		
	-- Responses
	local nResponseHeight = 0

	for idx, drResponse in ipairs(tResponseList) do
		local eResponseType = drResponse:GetType()
		local wndCurr = nil
		
		------------------------------------------------------------------------
		--Links the decline text and data to Decline button
		------------------------------------------------------------------------
		if eResponseType == DialogResponse.DialogResponseType_Goodbye then 
		
			self.wndMain:FindChild("Decline"):SetTooltip(drResponse:GetText())
			self.wndMain:FindChild("Decline"):SetData(drResponse)

		------------------------------------------------------------------------
		--Sets buttons for completeing quest
		------------------------------------------------------------------------
		elseif eResponseType == DialogResponse.DialogResponseType_QuestComplete then 
			IncompleteOnly = false
		
			-------------------------------------------------------------------------------------------
			--Sets the first reward to the Accept Button 
			-------------------------------------------------------------------------------------------
			if NumChoice == 0 then 
			
				self.wndMain:FindChild("Accept"):SetTooltip(drResponse:GetText())
				self.wndMain:FindChild("Accept"):SetData(drResponse)
				self.wndMain:FindChild("Accept"):Enable(true)
				self.wndMain:FindChild("Accept"):SetText("Complete")
				
				self.wndMain:FindChild("Conditional"):SetTooltip("This quest only has 1 reward but could not be completed for some reason. Possibly because you already have a copy of the unique reward.")
				self.wndMain:FindChild("Conditional"):SetData(drResponse)
				self.wndMain:FindChild("Conditional"):Enable(true)
				self.wndMain:FindChild("Conditional"):SetText("Close")
				
				First_drResponse = drResponse
				NumChoice = 1
				self.wndMain:FindChild("Accept_Decline"):FindChild("Conditional"):Show(true)
			-------------------------------------------------------------------------------------------
			--If there are more choices, removes the first reward and places the choices in the window
			-------------------------------------------------------------------------------------------
			elseif NumChoice == 1 then 
						
				self.wndMain:FindChild("Accept"):SetTooltip("")
				self.wndMain:FindChild("Accept"):SetData(nil)
				self.wndMain:FindChild("Accept"):Enable(false)
				self.wndMain:FindChild("Accept"):SetText("Pick One")
				
				wndCurr = Apollo.LoadForm("InstantQuestAccept.xml", "Reward", SelectWnd:FindChild("ResponseItemContainer"), self)
				self:HelperComputeRewardIcon(wndCurr, First_drResponse:GetRewardId(), queCurr:GetRewardData().arRewardChoices)
				wndCurr:FindChild("ResponseItemText"):SetData(First_drResponse:GetText())
				wndCurr:FindChild("ResponseItemText"):SetText(First_drResponse:GetText())
				wndCurr:FindChild("ResponseItemText"):SetTextColor(self:HelperComputeRewardTextColor(First_drResponse:GetRewardId(), DialogSys.GetViewableQuest(idQuest):GetRewardData()))
				wndCurr:FindChild("ResponseItemBtn"):SetData(First_drResponse)
				
				wndCurr = Apollo.LoadForm("InstantQuestAccept.xml", "Reward", SelectWnd:FindChild("ResponseItemContainer"), self)
				self:HelperComputeRewardIcon(wndCurr, drResponse:GetRewardId(), queCurr:GetRewardData().arRewardChoices)
				wndCurr:FindChild("ResponseItemText"):SetData(drResponse:GetText())
				wndCurr:FindChild("ResponseItemText"):SetText(drResponse:GetText())
				wndCurr:FindChild("ResponseItemText"):SetTextColor(self:HelperComputeRewardTextColor(drResponse:GetRewardId(), DialogSys.GetViewableQuest(idQuest):GetRewardData()))
				wndCurr:FindChild("ResponseItemBtn"):SetData(drResponse)
				
				nResponseHeight = nResponseHeight + (wndCurr:GetHeight() * 2)
				NumChoice = 2
				
				self.AcceptNow = false --Pauses the turn in to choose a reward, otherwise throws error because of "Pressing" Accept with a null value.
				self.wndMain:FindChild("Accept_Decline"):FindChild("Conditional"):Show(false)
				self.wndMain:FindChild("Accept_Decline"):FindChild("Accept"):Show(true)
				self.wndMain:FindChild("Accept_Decline"):FindChild("Decline"):Show(true)
			-------------------------------------------------------------------------------------------	
			--Places all choices after the second in the window
			-------------------------------------------------------------------------------------------
			else		
				
				wndCurr = Apollo.LoadForm("InstantQuestAccept.xml", "Reward", SelectWnd:FindChild("ResponseItemContainer"), self)
				self:HelperComputeRewardIcon(wndCurr, drResponse:GetRewardId(), queCurr:GetRewardData().arRewardChoices)
				wndCurr:FindChild("ResponseItemText"):SetData(drResponse:GetText())
				wndCurr:FindChild("ResponseItemText"):SetText(drResponse:GetText())
				wndCurr:FindChild("ResponseItemText"):SetTextColor(self:HelperComputeRewardTextColor(drResponse:GetRewardId(), DialogSys.GetViewableQuest(idQuest):GetRewardData()))
				wndCurr:FindChild("ResponseItemBtn"):SetData(drResponse)
				
				nResponseHeight = nResponseHeight + wndCurr:GetHeight()
				
			end
		
		------------------------------------------------------------------------
		-- Links the Accept info to the Accept Quest Button or places a View Quest button in 
		-- the Quest Selection Screen
		------------------------------------------------------------------------
		elseif eResponseType == DialogResponse.DialogResponseType_QuestAccept or
		 	   eResponseType == DialogResponse.DialogResponseType_ViewQuestAccept or
			   eResponseType == DialogResponse.DialogResponseType_ViewQuestComplete then
			
			IncompleteOnly = false
		
			self.wndMain:FindChild("Accept"):SetTooltip(drResponse:GetText())
			self.wndMain:FindChild("Accept"):SetData(drResponse)
			self.wndMain:FindChild("Accept"):SetText("Accept")
			
			local crTextColor = eResponseType == DialogResponse.DialogResponseType_QuestMoreInfo and CColor.new(128/255, 150/255, 168/255, 1) or CColor.new(1, 1, 1, 1)
			wndCurr = Apollo.LoadForm("InstantQuestAccept.xml", "QuestSelect", SelectWnd:FindChild("ResponseItemContainer"), self)
			wndCurr:FindChild("QuestSelectIcon"):SetSprite(self:HelperComputeIconPath(eResponseType))
			wndCurr:FindChild("QuestSelectText"):SetText(drResponse:GetText())
			wndCurr:FindChild("QuestSelectText"):SetTextColor(crTextColor)
			wndCurr:FindChild("QuestSelectBtn"):SetData(drResponse)
			
			nResponseHeight = nResponseHeight + wndCurr:GetHeight()
			
		elseif eResponseType == DialogResponse.DialogResponseType_ViewQuestIncomplete then
		
			local crTextColor = eResponseType == DialogResponse.DialogResponseType_QuestMoreInfo and CColor.new(128/255, 150/255, 168/255, 1) or CColor.new(1, 1, 1, 1)
			wndCurr = Apollo.LoadForm("InstantQuestAccept.xml", "IncompleteQuest", SelectWnd:FindChild("ResponseItemContainer"), self)
			wndCurr:FindChild("IncompleteQuestIcon"):SetSprite(self:HelperComputeIconPath(eResponseType))
			wndCurr:FindChild("IncompleteQuestText"):SetText(drResponse:GetText())
			wndCurr:FindChild("IncompleteQuestText"):SetTextColor(crTextColor)
			wndCurr:FindChild("IncompleteQuestBtn"):SetData(drResponse)
			
			nResponseHeight = nResponseHeight + wndCurr:GetHeight()
		end
	end
	
	if IncompleteOnly == true then					
		self.wndMain:FindChild("Accept"):SetText("Not used")
	end
	
	local nLeft, nTop, nRight, nBottom = SelectWnd:FindChild("ResponseItemContainer"):GetAnchorOffsets()
	SelectWnd:FindChild("ResponseItemContainer"):ArrangeChildrenVert(0, function(b,a) return b:FindChild("ResponseItemCantUse"):IsShown() end)
	SelectWnd:FindChild("ResponseItemContainer"):SetAnchorOffsets(nLeft, nTop, nRight, nTop + nResponseHeight)
	SelectWnd:ArrangeChildrenVert()
	
	self.wndMain:FindChild("Scroll"):RecalculateContentExtents()
			
	SelectWnd:FindChild("ResponseItemContainer"):Show(true)
	self.wndMain:FindChild("Accept_Decline"):Show(true)

	self.wndMain:Show(true)
	self.wndMain:FindChild("Selection"):Show(true)
	SelectWnd:Show(true)
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnSelectBtnClick(wndHandler, wndControl) -- QuestSelect Button
	
	if not wndHandler or not wndHandler:GetData() then
		return
	end
	
	self.AcceptNow = true -- Accept or complete quest after clicking the view quest button 
	
	wndHandler:GetData():Select()
end


---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnIncompleteQuestBtnClick( wndHandler, wndControl)
	self:OnDialog_Close()
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnChooseRewardClick(wndHandler, wndControl) -- Links the Selected reward to the Accept Button
	
	if not wndHandler or not wndHandler:GetData() then
		return
	end

	local drResponse = wndHandler:GetData()
	if drResponse:GetRewardId() and drResponse:GetRewardId() ~= -1 and drResponse:GetRewardId() ~= self.bRewardPicked then

		for idx, wndCurr in pairs(self.wndMain:FindChild("ResponseItemContainer"):GetChildren()) do
			if wndCurr:FindChild("ResponseItemText") and wndCurr:FindChild("ResponseItemText"):GetData() then
				wndCurr:FindChild("ResponseItemText"):SetText(wndCurr:FindChild("ResponseItemText"):GetData())
				wndCurr:FindChild("ResponseItemText"):SetTextColor(kcrDefaultOptionColor)
			end
		end
		
		self.bRewardPicked = drResponse:GetRewardId()
		
		wndHandler:FindChild("ResponseItemText"):SetTextColor(kcrHighlightOptionColor)
		self.wndMain:FindChild("Accept"):SetTooltip(String_GetWeaselString(Apollo.GetString("Dialog_TakeItem"), wndHandler:FindChild("ResponseItemText"):GetData()))
		self.wndMain:FindChild("Accept"):SetData(drResponse)
		self.wndMain:FindChild("Accept"):Enable(true)
		self.wndMain:FindChild("Accept"):SetText("Complete")
	else
	
		self.wndMain:Show(false)
	
		if self.AcceptAllSetting == false then
			self.AcceptNow = false
		else
			self.AcceptNow = true
		end
	
		wndHandler:GetData():Select() -- All the work is done in DialogSys's Select Method
	end
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnCloseBtnClick(wndHandler, wndControl) -- Clicked for Close Button

	if not wndHandler or not wndHandler:GetData() then
		return
	end

	self.wndMain:Show(false)
	
	wndHandler:GetData():Select() -- All the work is done in DialogSys's Select Method
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnAcceptBtnClick(wndHandler, wndControl) -- Clicked for Accept Button

	if not wndHandler or not wndHandler:GetData() then
		return
	end
	
	self.wndMain:Show(false)
	
	if self.AcceptAllSetting == false then
		self.AcceptNow = false
	else
		self.AcceptNow = true
	end
	
	wndHandler:GetData():Select() -- All the work is done in DialogSys's Select Method
end

---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
-- Helpers
---------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------
function InstantQuestAccept:HelperComputeIconPath(eResponseType)
	local strSprite = "CRB_DialogSprites:sprDialog_Icon_Decline"
	if eResponseType == DialogResponse.DialogResponseType_ViewVending then
		strSprite = "sprMM_VendorGeneral"
	elseif eResponseType == DialogResponse.DialogResponseType_ViewTraining then
		strSprite = "sprMM_VendorGeneral"
	elseif eResponseType == DialogResponse.DialogResponseType_ViewCraftingStation then
		strSprite = "sprMM_VendorGeneral"
	elseif eResponseType == DialogResponse.DialogResponseType_ViewTradeskillTraining then
		strSprite = "sprMM_VendorGeneral"
	elseif eResponseType == DialogResponse.DialogResponseType_ViewQuestAccept then
		strSprite = "CRB_DialogSprites:sprDialog_Icon_Exclamation"
	elseif eResponseType == DialogResponse.DialogResponseType_ViewQuestComplete then
		strSprite = "CRB_DialogSprites:sprDialog_Icon_Check"
	elseif eResponseType == DialogResponse.DialogResponseType_ViewQuestIncomplete then
		strSprite = "CRB_DialogSprites:sprDialog_Icon_DisabledCheck"
	elseif eResponseType == DialogResponse.DialogResponseType_Goodbye then
		strSprite = "CRB_DialogSprites:sprDialog_Icon_Decline"
	elseif eResponseType == DialogResponse.DialogResponseType_QuestAccept then
		strSprite = "CRB_DialogSprites:sprDialog_Icon_Exclamation"
	elseif eResponseType == DialogResponse.DialogResponseType_QuestMoreInfo then
		strSprite = "CRB_DialogSprites:sprDialog_Icon_More"
	elseif eResponseType == DialogResponse.DialogResponseType_QuestComplete then
		strSprite = "CRB_DialogSprites:sprDialog_Icon_Check"
	end
	return strSprite
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:HelperComputeRewardTextColor(idReward, tChoiceRewardData)
	if idReward == -1 then
		return kcrDefaultOptionColor
	end

	for idx, tCurrReward in ipairs(tChoiceRewardData) do
		if tCurrReward and tCurrReward.idReward == idReward then
			if tCurrReward.eType == Quest.Quest2RewardType_Item then
				return karEvalColors[tCurrReward.itemReward:GetItemQuality()]
			end
			break
		end
	end
	
	return kcrDefaultOptionColor
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:HelperComputeRewardIcon(wndCurr, idReward, tChoiceRewardData)
	if idReward == -1 then
		return
	end

	local tFoundRewardData = nil
	for idx, tCurrReward in ipairs(tChoiceRewardData) do
		if tCurrReward.idReward == idReward then
			tFoundRewardData = tCurrReward
			break
		end
	end

	if tFoundRewardData and wndCurr then
		local strIconSprite = ""
		if tFoundRewardData.eType == Quest.Quest2RewardType_Item then
			strIconSprite = tFoundRewardData.itemReward:GetIcon()
			self:HelperBuildItemTooltip(wndCurr, tFoundRewardData.itemReward)
		elseif tFoundRewardData.eType == Quest.Quest2RewardType_Reputation then
			strIconSprite = "Icon_ItemMisc_UI_Item_Parchment"
			wndCurr:SetTooltip(String_GetWeaselString(Apollo.GetString("Dialog_FactionRepReward"), tFoundRewardData.nAmount, tFoundRewardData.strFactionName))
		elseif tFoundRewardData.eType == Quest.Quest2RewardType_TradeSkillXp then
			strIconSprite = "ClientSprites:Icon_ItemMisc_tool_0001"
			wndCurr:SetTooltip(String_GetWeaselString(Apollo.GetString("Dialog_TradeskillXPReward"), tFoundRewardData.nXP, tFoundRewardData.strTradeskill)) --hardcoded
		elseif tFoundRewardData.eType == Quest.Quest2RewardType_GrantTradeskill then
			strIconSprite = "ClientSprites:Icon_ItemMisc_tool_0001"
			wndCurr:SetTooltip("")
		elseif tFoundRewardData.eType == Quest.Quest2RewardType_Money then
			if tFoundRewardData.eCurrencyType == Money.CodeEnumCurrencyType.Credits then
				local strText = ""
				local nInCopper = tFoundRewardData.nAmount
				if nInCopper >= 1000000 then
					strText = strText .. String_GetWeaselString(Apollo.GetString("CRB_Platinum"), math.floor(nInCopper / 1000000))
				end
				if nInCopper >= 10000 then
					strText = strText .. String_GetWeaselString(Apollo.GetString("CRB_Gold"), math.floor(nInCopper % 1000000 / 10000))
				end
				if nInCopper >= 100 then
					strText = strText .. String_GetWeaselString(Apollo.GetString("CRB_Silver"), math.floor(nInCopper % 10000 / 100))
				end
				wndCurr:SetTooltip(strText .. String_GetWeaselString(Apollo.GetString("CRB_Copper"), math.floor(nInCopper % 100)))
				strIconSprite = "ClientSprites:Icon_ItemMisc_bag_0001"
			else
				local tDenomInfo = GameLib.GetPlayerCurrency(tFoundRewardData.eCurrencyType):GetDenomInfo()
				if tDenomInfo ~= nil then
					strText = tFoundRewardData.nAmount .. " " .. tDenomInfo[1].strName
					strIconSprite = "ClientSprites:Icon_ItemMisc_bag_0001"
					wndCurr:SetTooltip(strText)				
				end	
			end
		end

		wndCurr:FindChild("ResponseItemIcon"):Show(false)
		wndCurr:FindChild("ResponseItemRewardBG"):Show(true)
		wndCurr:FindChild("ResponseItemRewardIcon"):SetSprite(strIconSprite)
		wndCurr:FindChild("ResponseItemCantUse"):Show(self:HelperPrereqFailed(tFoundRewardData.itemReward))
	end
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:HelperDrawLootItem(wndCurrReward, tCurrReward)
	local strIconSprite = ""
	if tCurrReward and tCurrReward.eType == Quest.Quest2RewardType_Item then

		strIconSprite = tCurrReward.itemReward:GetIcon()
		wndCurrReward:FindChild("LootItemCantUse"):Show(self:HelperPrereqFailed(tCurrReward.itemReward))
		self:HelperBuildItemTooltip(wndCurrReward, tCurrReward.itemReward)

	elseif tCurrReward and tCurrReward.eType == Quest.Quest2RewardType_Reputation then

		-- Reputation has overloaded fields: objectId is factionId. objectAmount is rep amount.
		strIconSprite = "Icon_ItemMisc_UI_Item_Parchment"
		wndCurrReward:FindChild("LootDescription"):SetText(String_GetWeaselString(Apollo.GetString("Dialog_FactionRepReward"), tCurrReward.nAmount))
		wndCurrReward:SetTooltip(String_GetWeaselString(Apollo.GetString("Dialog_FactionRepReward"), tCurrReward.nAmount, tCurrReward.strFactionName))

	elseif tCurrReward and tCurrReward.eType == Quest.Quest2RewardType_TradeSkillXp then

		-- Tradeskill XP has overloaded fields: objectId is factionId. objectAmount is rep amount.
		strIconSprite = "ClientSprites:Icon_ItemMisc_tool_0001"
		wndCurrReward:FindChild("LootDescription"):SetText(String_GetWeaselString(Apollo.GetString("Dialog_TradeskillXPReward"), tCurrReward.nXP, tCurrReward.strTradeskill))
		wndCurrReward:SetTooltip(String_GetWeaselString(Apollo.GetString("Dialog_TradeskillXPReward"), tCurrReward.nXP, tCurrReward.strTradeskill))

	elseif tCurrReward and tCurrReward.eType == Quest.Quest2RewardType_GrantTradeskill then

		-- Tradeskill XP has overloaded fields: objectId is tradeskillId.
		strIconSprite = "ClientSprites:Icon_ItemMisc_tool_0001"
		wndCurrReward:FindChild("LootDescription"):SetText(tCurrReward.strTradeskill)
		wndCurrReward:SetTooltip("")

	elseif tCurrReward and tCurrReward.eType == Quest.Quest2RewardType_Money then
		if tCurrReward.eCurrencyType == Money.CodeEnumCurrencyType.Credits then
			strIconSprite = "ClientSprites:Icon_ItemMisc_bag_0001"
			wndCurrReward:FindChild("LootDescription"):Show(false)
			wndCurrReward:FindChild("LootCashWindow"):Show(true)
			wndCurrReward:FindChild("LootCashWindow"):SetAmount(tCurrReward.nAmount, 0)

			local strText = ""
			local nInCopper = tCurrReward.nAmount
			if nInCopper >= 1000000 then
				strText = strText .. String_GetWeaselString(Apollo.GetString("CRB_Platinum"), math.floor(nInCopper / 1000000))
			end
			if nInCopper >= 10000 then
				strText = strText .. String_GetWeaselString(Apollo.GetString("CRB_Gold"), math.floor(nInCopper % 1000000 / 10000))
			end
			if nInCopper >= 100 then
				strText = strText .. String_GetWeaselString(Apollo.GetString("CRB_Silver"), math.floor(nInCopper % 10000 / 100))
			end
			wndCurrReward:SetTooltip(strText .. String_GetWeaselString(Apollo.GetString("CRB_Copper"), math.floor(nInCopper % 100)))
		else
			local tDenomInfo = GameLib.GetPlayerCurrency(tCurrReward.eCurrencyType):GetDenomInfo()
			if tDenomInfo ~= nil then
				strText = self:FormatCurrency(tCurrReward.nAmount, tDenomInfo)
				strIconSprite = "ClientSprites:Icon_ItemMisc_bag_0001"
				wndCurrReward:FindChild("LootDescription"):Show(false)
				wndCurrReward:FindChild("LootCashWindow"):Show(true)
				wndCurrReward:FindChild("LootCashWindow"):SetMoneySystem(tCurrReward.eCurrencyType or Money.CodeEnumCurrencyType.Credits)				
				wndCurrReward:FindChild("LootCashWindow"):SetAmount(tCurrReward.nAmount, 0)				
				wndCurrReward:SetTooltip(strText)				
			end
		end
	end

	wndCurrReward:FindChild("LootItemIcon"):SetSprite(strIconSprite)
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnGenerateTooltip(wndHandler, wndControl, eType, arg1, arg2)
	local xml = nil
	if eType == Tooltip.TooltipGenerateType_ItemData then
		local itemCurr = arg1
		local itemEquipped = itemCurr:GetEquippedItemForItemType()

		Tooltip.GetItemTooltipForm(self, wndControl, itemCurr, {bPrimary = true, bSelling = self.bVendorOpen, itemCompare = itemEquipped})

	elseif eType == Tooltip.TooltipGenerateType_Reputation or eType == Tooltip.TooltipGenerateType_TradeSkill then
		xml = XmlDoc.new()
		xml:StartTooltip(Tooltip.TooltipWidth)
		xml:AddLine(arg1)
		wndControl:SetTooltipDoc(xml)
	elseif eType == Tooltip.TooltipGenerateType_Money then
		xml = XmlDoc.new()
		xml:StartTooltip(Tooltip.TooltipWidth)
		xml:AddLine(arg1:GetMoneyString(), CColor.new(1, 1, 1, 1), "CRB_InterfaceMedium")
		wndControl:SetTooltipDoc(xml)
	else
		wndControl:SetTooltipDoc(nil)
	end
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:OnWindowClosed(wndHandler, wndControl) -- The 'esc' key from xml
	if wndHandler:GetId() ~= wndControl:GetId() then return end
	DialogSys.End()
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:HelperBuildItemTooltip(wndArg, item)
	wndArg:SetTooltipDoc(nil)
	wndArg:SetTooltipDocSecondary(nil)
	local itemEquipped = item:GetEquippedItemForItemType()
	Tooltip.GetItemTooltipForm(self, wndArg, item, {bPrimary = true, bSelling = false, itemCompare = itemEquipped})
end

--------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------
function InstantQuestAccept:HelperPrereqFailed(itemCurr)
	return itemCurr and itemCurr:IsEquippable() and not itemCurr:CanEquip()
end

---------------------------------------------------------------------------------------------------
-- Configure Functions
---------------------------------------------------------------------------------------------------

function InstantQuestAccept:OnAcceptAllSetting( wndHandler, wndControl, eMouseButton )
	if wndControl:IsChecked() then
		self.AcceptAllSetting = true
		self.AcceptNow = true
	else
		self.AcceptAllSetting = false
		self.AcceptNow = false
	end
end

function InstantQuestAccept:OnPostSetting( wndHandler, wndControl, eMouseButton )
	if wndControl:IsChecked() then
		self.PostLog = true
	else
		self.PostLog = false
	end
end

function InstantQuestAccept:OnConfig_Close()
	self.Conf:Close()
end

function InstantQuestAccept:ResetLoc( wndHandler, wndControl, eMouseButton )
	local tMax = Apollo.GetDisplaySize()
	self.wndMain:SetAnchorOffsets(tMax.nWidth - 450, 50, tMax.nWidth - 50, 650)
end

---------------------------------------------------------------------------------------------------
-- MainWindow Functions
---------------------------------------------------------------------------------------------------

function InstantQuestAccept:MainMoved( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom )
	
end

---------------------------------------------------------------------------------------------------
-- Dialog instance
---------------------------------------------------------------------------------------------------
local DialogInst = InstantQuestAccept:new()
DialogInst:Init()
